<#
    .SYNOPSIS
    Script to generate a CSV report displaying agent primary and failover management server assignments.

    .DESCRIPTION
    This script is collects primary and failover management server information for every agent in an Operations Manager management group. Afterwards, it outputs a CSV file containing that data into the same folder as the script.
    
    This script leverages the SCOM SDK rather than PowerShell cmdlets for compatibility. This script must be ran on a computer or server with an Operations Manager console or management server role installed.

    This script supports the -Verbose and -Debug switches. Use -Verbose to view detailed progress information.

    .PARAMETER OMServer
    Specifies the name of the Operations Manager server to which the script should connect in order to retrieve agent information. Default value is the local computer's name.

    .PARAMETER OutputFileName
    Specifies the desired filename for the output report. Default value is "AgentFailoverReport". This value will have the current date/time appended to its front and will be saved as a .csv file.

    .PARAMETER OutputLocation
    Specifies the folder in which to save the output report. Default value is the current folder.

    .NOTES
    Written by Gabriel Taylor, 2014
#>


[CmdletBinding()]
param(
    [string]$OMServer = $env:COMPUTERNAME,
    [string]$OutputFilename = "AgentFailoverReport",
    [string]$OutputFolder = ".\"
)

## Start Timer
$StartTime = Get-Date

## Load SCOM SDK
Write-Verbose "[$(Get-Date)] Loading SCOM SDK ..."
[System.Reflection.Assembly]::LoadWithPartialName("Microsoft.EnterpriseManagement.OperationsManager.Common") > $null
[System.Reflection.Assembly]::LoadWithPartialName("Microsoft.EnterpriseManagement.OperationsManager") > $null

## Connect to SCOM management group
Write-Verbose "[$(Get-Date)] Connecting to SCOM management group on $OMServer ..."
$MGConnSetting = New-Object Microsoft.EnterpriseManagement.ManagementGroupConnectionSettings($OMServer)
$MG = New-Object Microsoft.EnterpriseManagement.ManagementGroup($OMServer)
Write-Verbose "[(Get-Date)] Connection successfully established."

## Collect agents
Write-Verbose "[$(Get-Date)] Retrieving all SCOM Agents ..."
$Administration = $MG.Administration
$AgentList = @()
$AgentList = $Administration.GetAgentManagedComputers() | Sort-Object -Property Name
[int]$AgentCount = $AgentList.count
Write-Verbose "[$(Get-Date)] SCOM Agents successfully retrieved."

## Initialize an array in which to store custom agent data objects
$AgentDataObjs = @()

## Retrieve agent primary and failover management server information and store it within a custom object
Write-Verbose "[$(Get-Date)] $AgentCount agents to process ..."
foreach($Agent in $AgentList) {
    Write-Verbose "[$(Get-Date)] Now processing agent $($Agent.Name)`..."
    
    ## Collect agent properties and store within variables
    $AgentName = $Agent.Name
    Write-Verbose "[$(Get-Date)] Getting $AgentName Primary Management Server ..."
    $AgentPrimaryServer = $Agent.PrimaryManagementServerName
    Write-Verbose "[$(Get-Date)] Getting $AgentName Failover Management Servers ..."
    $AgentFailoverServers = ($Agent.GetFailoverManagementServers()).Name | Sort-Object
    $AgentFailoverServerCount = $AgentFailoverServers.count
    $AgentFailoverServersJoined = $AgentFailoverServers -join ";"
    Write-Verbose "[$(Get-Date)] Creating agent data object ..."
    
    ## Create the custom agent data object
    $AgentDataObj = New-Object -TypeName PSObject -Property @{
        Server_Name = $AgentName
        Primary_MS = $AgentPrimaryServer
        Failover_MSCount = $AgentFailoverServerCount
        Failover_MSs = $AgentFailoverServersJoined
    }
    
    ## Add the agent data object to the array
    Write-Verbose "[$(Get-Date)] Adding agent data object $($Agent.DNSHostName)` to array ..."
    $AgentDataObjs += $AgentDataObj
    [int]$AgentCount = $AgentCount - 1
    Write-Verbose "[$(Get-Date)] $AgentCount agents remaining ..."
}

## Format the output filename
$OutputFolder = (Get-Item -Path $OutputFolder).FullName
$CSVFileName = $OutputFolder + "\" + ((Get-Date).ToString("yyyy-MM-dd HH:mm")) + " " + $OutputFilename + ".csv"

## Export the CSV
Write-Verbose "[$(Get-Date)] Exporting CSV ..."
$AgentDataObjs | Sort-Object -Property Server_Name | Select-Object -Property Server_Name,Primary_MS,Failover_MSCount,Failover_MSs | Export-Csv -NoTypeInformation -Path $CSVFileName

## Stop the timer
$EndTime = Get-Date
$ProcessTime = New-TimeSpan -Start $StartTime -End $EndTime

## Say goodnight, Gracie
Write-Verbose "[$(Get-Date)] Script finished; results output to $CSVFileName. Process ran for $ProcessTime."
